/**
 * jquery.trumpet.js
 *
 * @version 1.0.0
 * @author  Arran Jacques
 * @license The MIT License (MIT) - http://opensource.org/licenses/MIT
 * @copyright (c) 2014 Arran Jacques
 */

(function(window, document, $) {

    'use strict';

    /* =============================================================================
     * Extend the main jQuery object.
     *
     *
     */

    /**
     * Throw a simple alert to the user’s browser’s window.
     *
     * @param   Object
     *          An object literal that defines the values of the settings for the
     *          alert box.
     */
    $.alert =  function(s) {
        var settings = (typeof s === 'object') ? s : {};
        var alert = new TrumpetAlert({
            style: (typeof settings.style === 'string') ? settings.style: false,
            coverBackground: (typeof settings.coverBackground === 'boolean') ? settings.coverBackground : false,
            title: (typeof settings.title === 'string') ? settings.title : 'Alert',
            message: (typeof settings.message === 'string') ? settings.message : false,
            acceptButton: (typeof settings.acceptButton === 'string') ? settings.acceptButton : 'OK',
            onAccept: (typeof settings.onAccept === 'function') ? settings.onAccept : function() {}          
        });
        alert.display();
    };

    /**
     * Throw a confirm box to the user’s browser’s window.
     *
     * @param   Object
     *          An object literal that defines the values of the settings for the
     *          confim box.
     */
    $.confirm =  function(s) {
        var settings = (typeof s === 'object') ? s : {};
        var confirm = new TrumpetConfirm({
            style: (typeof settings.style === 'string') ? settings.style: false,
            coverBackground: (typeof settings.coverBackground === 'boolean') ? settings.coverBackground : false,
            title: (typeof settings.title === 'string') ? settings.title : 'Confirm',
            message: (typeof settings.message === 'string') ? settings.message : false,
            confirmButton: (typeof settings.confirmButton === 'string') ? settings.confirmButton : 'Confirm',
            cancelButton: (typeof settings.cancelButton === 'string') ? settings.cancelButton : 'Cancel',
            onConfirm: (typeof settings.onConfirm === 'function') ? settings.onConfirm : function() {},
            onCancel: (typeof settings.onCancel === 'function') ? settings.onCancel : function() {}
        });
        confirm.display();
    };

    /**
     * Throw a confirm box to the user’s browser’s window that requires the user to
     * enter a string before they can confirm.
     *
     * @param   Object
     *          An object literal that defines the values of the settings for the
     *          confim box.
     */
    $.activeConfirm =  function(s) {
        var settings = (typeof s === 'object') ? s : {};
        var confirm = new TrumpetActiveConfirm({
            style: (typeof settings.style === 'string') ? settings.style: false,
            coverBackground: (typeof settings.coverBackground === 'boolean') ? settings.coverBackground : false,
            title: (typeof settings.title === 'string') ? settings.title : 'Confirm',
            message: (typeof settings.message === 'string') ? settings.message : false,
            confirmText: (typeof settings.confirmText === 'string') ? settings.confirmText : 'CONFIRM',
            confirmButton: (typeof settings.confirmButton === 'string') ? settings.confirmButton : 'Confirm',
            cancelButton: (typeof settings.cancelButton === 'string') ? settings.cancelButton : 'Cancel',
            onConfirm: (typeof settings.onConfirm === 'function') ? settings.onConfirm : function() {},
            onCancel: (typeof settings.onCancel === 'function') ? settings.onCancel : function() {}
        });
        confirm.display();
    };

    /* =============================================================================
     * Extend the jQuery prototype object with methods that directly map to
     * corresponding methods on the main jQuery object.
     *
     *
     */

    $.fn.extend({

        /**
         * Maps to $.alert(s).
         *
         * @param   String
         *          jQuery selector string.
         *
         * @param   Object
         *          An object literal that defines the values of the settings for the
         *          alert box.
         */
        alert: function(q, s) {
            $(this).each(function() {
                $(this).on(q, function() {
                    var settings = (typeof s === 'object') ? s : {};
                    $.alert(settings);
                });
            });
        },

        /**
         * Maps to $.confirm(s).
         *
         * @param   String
         *          jQuery selector string.
         *
         * @param   Object
         *          An object literal that defines the values of the settings for the
         *          confim box.
         */
        confirm: function(q, s) {
            $(this).each(function() {
                $(this).on(q, function() {
                    var settings = (typeof s === 'object') ? s : {};
                    $.confirm(settings);
                });
            });
        },

        /**
         * Maps to $.activeConfirm(s).
         *
         * @param   String
         *          jQuery selector string.
         *
         * @param   Object
         *          An object literal that defines the values of the settings for the
         *          confim box.
         */
        activeConfirm: function(q, s) {
            $(this).each(function() {
                $(this).on(q, function() {
                    var settings = (typeof s === 'object') ? s : {};
                    $.activeConfirm(settings);
                });
            });
        },
    });

    /* =============================================================================
     * Trumpet alert objects and helper methods for building them.
     *
     *
     */

    // Keep track of the total number of alert boxes that have been appended to the
    // window.
    window._trumpetAlertBoxes = 0;

    // Methods for building alert boxes.
    var build = {

        /**
         * Return a string a HTML for a trumpet alert box.
         *
         * @param   Object
         *          An object literal that defines the values of the settings that will
         *          be used to build the HTML.
         *
         * @return  String
         */
        html: function(s) {

            // Generate a random 10 character URI for the alert.
            var uri = '_',
                possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
            for(var i = 0; i < 10; i++){
                uri += possible.charAt(Math.floor(Math.random() * possible.length));
            }

            // Build vars that will be used to create the HTML.
            var
                z_index = 10000 + window._trumpetAlertBoxes,
                style_class = (s.style) ? ' _trumpetAlert--' + s.style : '',
                cover_background = (s.coverBackground) ? ' _trumpetAlert__cover--bkg' : '';

            // Start the HTML for the alert.
            var html = '';
            html += '<div style="z-index:' + z_index + ';" class="_trumpetAlert__cover' + style_class + cover_background + '" id="' + uri + '">';
            html += '<div class="_trumpetAlert__cell' + style_class + '"><div class="_trumpetAlert__wrap' + style_class + '">';
            html += '<div class="_trumpetAlert' + style_class + '">';
            html += '<h1 class="_trumpetAlert__title' + style_class + '">' + s.title + '</h1>';

            // If a message has been set then add the message to the alert HTML.
            if (s.message) {
                html += '<div class="_trumpetAlert__message' + style_class + '">' + s.message + '</div>';
            }

            // If the alert is an active confirm type then add the necessary input and label
            // to the alert HTML.
            if (s.type === 'active-confirm') {
                html += '<div class="_trumpetAlert__input__wrap' + style_class + '">';
                html += '<form class="_trumpetAlert__form' + style_class + '" id="' + uri + '-form">';
                html += '<label for="' + uri + '-input" class="_trumpetAlert__label' + style_class + '">';
                html += 'Type the word "' + s.confirmText + '" and then click "' + s.confirmButton + '" to proceed.';
                html += '</label>';
                html += '<input type="text" class="_trumpetAlert__input' + style_class + '" id="' + uri + '-input" />';                
                html += '</form></div>';
            }

            // Add the appropriate control buttons to the HTML, which will depend on the
            // type of alert being thrown.
            switch (s.type) {
                case 'alert':
                    html += '<div class="_trumpetAlert__controls' + style_class + '">';
                    html += '<span class="_trumpetAlert__button' + style_class + '" id="' + uri + '-ok">' + s.acceptButton + '</span>';
                    html += '</div>';
                    break;
                case 'confirm':
                case 'active-confirm':
                    html += '<div class="_trumpetAlert__controls' + style_class + '">';
                    html += '<span class="_trumpetAlert__button' + style_class + '" id="' + uri + '-cancel">' + s.cancelButton + '</span>';
                    html += '<span class="_trumpetAlert__button' + style_class + '" id="' + uri + '-confirm">' + s.confirmButton + '</span>';
                    html += '</div>';
                    break;
            }

            // Close any open HTML tags for the alert.
            html += '</div></div></div></div>';

            return html;
        }
    };

    /**
     * An alert box.
     *
     * @param   Object
     *          An object literal that defines the values of the settings for the
     *          alert box.
     */
    function TrumpetAlert(s) {

        var _self = this;

        // Build the html for the alert box.
        this.html = build.html({
            style: s.style,
            type: 'alert',
            coverBackground: s.coverBackground,
            title: s.title,
            message: s.message,
            acceptButton: s.acceptButton
        });

        // Get the alert box's unique URI.
        this.uri = $(this.html).prop('id');

        /**
         * Append the alert box to the browser window.
         */
        this.display = function() {
            
            // Append the confirm box to the browser window.
            $('body').prepend(_self.html);

            // Initialise the confirm box’s controls.
            var box = $('#' + _self.uri);
            box.find('#' + _self.uri + '-ok').click(function() {
                box.remove();
                s.onAccept();
            });

            window._trumpetAlertBoxes++;
        };
    }

    /**
     * A confirm box.
     *
     * @param   Object
     *          An object literal that defines the values of the settings for the
     *          confirm box.
     */
    function TrumpetConfirm(s) {

        var _self = this;

        // Build the html for the confirm box.
        this.html = build.html({
            style: s.style,
            type: 'confirm',
            coverBackground: s.coverBackground,
            title: s.title,
            message: s.message,
            confirmButton: s.confirmButton,
            cancelButton: s.cancelButton
        });

        // Get the confirm box's unique URI.
        this.uri = $(this.html).prop('id');

        /**
         * Append the confirm box to the browser window.
         */
        this.display = function() {

            // Append the confirm box to the browser window.
            $('body').prepend(_self.html);

            // Initialise the confirm box’s controls.
            var box = $('#' + _self.uri);
            box.find('#' + _self.uri + '-cancel').click(function() {
                box.remove();
                s.onCancel();
            });
            box.find('#' + _self.uri + '-confirm').click(function() {
                box.remove();
                s.onConfirm();
            });

            window._trumpetAlertBoxes++;
        };
    }

    /**
     * A confirm box that requires the user to enter a string before they can
     * confirm.
     *
     * @param   Object
     *          An object literal that defines the values of the settings for the
     *          confirm box.
     */
    function TrumpetActiveConfirm(s) {

        var _self = this;

        // Build the html for the confirm box.
        this.html = build.html({
            style: s.style,
            type: 'active-confirm',
            coverBackground: s.coverBackground,
            title: s.title,
            message: s.message,
            confirmText: s.confirmText,
            confirmButton: s.confirmButton,
            cancelButton: s.cancelButton
        });

        // Get the alert box's unique URI.
        this.uri = $(this.html).prop('id');

        /**
         * Append the confirm box to the browser window and initialise it.
         */
        this.display = function() {

            // Append the confirm box to the browser window.
            $('body').prepend(_self.html);

            // Initialise the confirm box’s controls.
            var box = $('#' + _self.uri);
            box.find('#' + _self.uri + '-input').focus();
            box.find('#' + _self.uri + '-cancel').click(function() {
                box.remove();
                s.onCancel();
            });
            box.find('#' + _self.uri + '-confirm').click(function() {
                if (box.find('#' + _self.uri + '-input').val() === s.confirmText) {
                    box.remove();
                    s.onConfirm();
                } else {
                    box.find('#' + _self.uri + '-input').addClass('_trumpetAlert__input--fail');
                    box.find('#' + _self.uri + '-input').focus();
                }
            });
            box.find('#' + _self.uri + '-form').on('submit', function(e) {
                e.preventDefault();
                box.find('#' + _self.uri + '-confirm').click();
            });

            window._trumpetAlertBoxes++;
        };
    }

    // Once the DOM has loaded append a div to the DOM with a preload class. This
    // will cause the browser to load any background-images that are defined in that
    // CSS rules for that class.
    $(document).ready(function() {
        var p = '<div class="__preloadTrumpetAlertImage"></div>';
        $(p).find('.__preloadTrumpetAlertImage').css({
            'position': 'absolute',
            'top': '-99999',
            'left': '-9999px',
            'width': '2px',
            'height': '2px'
        });
        $('body').prepend(p);
    });

})(window, document, jQuery);